import java.awt.*;
import java.awt.color.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.nio.ByteBuffer;
import java.io.File;
import java.io.IOException;
import javax.swing.*;
import javax.media.opengl.*;
import javax.media.opengl.glu.*;
import javax.imageio.*;

public class Demo extends Frame implements GLEventListener, MouseListener, MouseMotionListener, MouseWheelListener {
  public static void main(String[] args) throws IOException {
    new Demo().setVisible(true);
  }

  final ByteBuffer imgRGBA;
  final BufferedImage img;

  GLCanvas panel;

  public Demo() throws IOException {

    panel = new GLCanvas();
    add(panel);

    BufferedImage i = ImageIO.read(new File("cc.png"));
    WritableRaster raster = Raster.createInterleavedRaster(
      DataBuffer.TYPE_BYTE, i.getWidth(), i.getHeight(), 4, null);
    ComponentColorModel colorModel=	new ComponentColorModel(
      ColorSpace.getInstance(ColorSpace.CS_sRGB), new int[] {8,8,8,8},
      true, false, ComponentColorModel.TRANSLUCENT, DataBuffer.TYPE_BYTE);			
    img =  new BufferedImage (colorModel, raster, false, null);
    
    Graphics2D g = img.createGraphics();
    AffineTransform gt = new AffineTransform();
    gt.translate (0, i.getHeight());
    gt.scale (1.0, -1.0);
    g.transform (gt);
    g.drawImage (i, null, null);
    g.dispose();

    DataBufferByte buf = (DataBufferByte)raster.getDataBuffer();
    imgRGBA = ByteBuffer.wrap(buf.getData());

    panel.addGLEventListener(this);
    panel.addMouseListener(this);
    panel.addMouseMotionListener(this);
    panel.addMouseWheelListener(this);

    setSize(600,600);
  }

  private double x = 0;
  private double y = 0;

  private GLU glu;  

  public void init(GLAutoDrawable drawable) {
    System.out.println ("init()");

    GL gl = drawable.getGL(); 
    gl.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f); // black 

    glu = new GLU();

    gl.glMatrixMode(GL.GL_PROJECTION);
    gl.glLoadIdentity();
    glu.gluOrtho2D(0, panel.getWidth(), 0, panel.getHeight());
  }

  public void display(GLAutoDrawable drawable) {
    System.out.println ("display()");
    GL gl = drawable.getGL();
    gl.glClear (GL.GL_COLOR_BUFFER_BIT);  // clear to clear color

    System.out.println(x + "," + y);

/*
    gl.glMatrixMode(GL.GL_MODELVIEW);
    gl.glLoadIdentity();
    glu.gluLookAt(x, y, 1.0,
                  x, y, 0.0,
                  0.0, 1.0, 0.0);
 
    gl.glRasterPos2d(0,0);
*/

    gl.glMatrixMode(GL.GL_MODELVIEW);
    gl.glLoadIdentity();
    gl.glScaled(scale, scale, 0.0);
System.out.println(scale);

//    gl.glRasterPos2d(-x,-y);
    gl.glRasterPos2d(0,0);
    gl.glBitmap(0,0,0.0f,0.0f,(float)-x,(float)-y,null);

    gl.glDrawPixels(img.getWidth(), img.getHeight(), gl.GL_RGBA,
      gl.GL_UNSIGNED_BYTE, imgRGBA);
  }

  public void reshape(GLAutoDrawable drawable,
                      int x, int y, int width, int height) {
    System.out.println ("reshape()");

    GL gl = drawable.getGL(); 
    gl.glViewport(0, 0, width, height); 

//    gl.glMatrixMode(GL.GL_PROJECTION);  
//    gl.glLoadIdentity();
//    glu.gluOrtho2D(0.0, width, 0.0, height);

//    gl.glMatrixMode(GL.GL_MODELVIEW);
//    gl.glLoadIdentity();
  }

  public void displayChanged(GLAutoDrawable drawable,
                             boolean modeChanged,
                             boolean deviceChanged) {
  }

  public void mouseClicked(MouseEvent e) { }
  public void mouseEntered(MouseEvent e) { }
  public void mouseExited(MouseEvent e) { }

  public void mousePressed(MouseEvent e) {
    opos = e.getPoint();
    opos.y = panel.getHeight() - opos.y - 1;
    System.out.println("new anchor: " + opos.x + "," + opos.y);
  }

  public void mouseReleased(MouseEvent e) {
    opos = null;
    System.out.println("cleared anchor");
  }

  
  private Point opos;

  public void mouseDragged(MouseEvent e) {
    final Point npos = e.getPoint();
    npos.y = panel.getHeight() - npos.y - 1;

    System.out.println("drag: " + (npos.x - opos.x) + "," + (npos.y - opos.y)); 

    x += (opos.x - npos.x);
    y += (opos.y - npos.y);
   
    opos = npos; 
    panel.repaint();

//    System.out.println(e.getX() + "," + (panel.getHeight() - e.getY()));

/*
    if (opos == null) opos = npos;
    else {
      x += opos.x - npos.x;
      y += npos.y - opos.y;
//      mpos = e.getPoint();
//      mpos.x = e.getX();
//    y = getHeight() - e.getY();
      opos = npos;
      panel.repaint();
    }
*/
  }

  public void mouseMoved(MouseEvent e) { }

  private double scale = 1.0;

  public void mouseWheelMoved(MouseWheelEvent e) {
    System.out.println(e.getWheelRotation());
    scale += e.getWheelRotation()/10.0;
    panel.repaint();
  }
}
